{+--------------------------------------------------------------------------+
 | Unit:        mwIPos
 | Created:     8.97
 | Author:      Martin Waldenburg
 | Copyright    1997, all rights reserved.
 | Description: Two simple and effective case insensitive "Pos" functions.
 |              The typecast in the parameters is a bit uncomfortable, but
 |              otherwise it would be significant slower.
 | Version:     1.6
 | Status:      FreeWare
 | Disclaimer:
 | This is provided as is, expressly without a warranty of any kind.
 | You use it at your own risc.
 +--------------------------------------------------------------------------+}

unit mwIPos;

interface
uses
  SysUtils;
procedure MakeCharTable;
function IPos(Pat, Text: PChar):Integer;
function IPosE(Pat, Text: PChar; CharPos: LongInt):Integer;
var
  CharTable:array[#0..#255] of byte;

implementation

procedure MakeCharTable;
var
  I: Integer;
begin
  for I:= 0 to 255 do
    begin
      If (I > 64) and (I < 91) then CharTable[Char(I)]:= I + 32
      else CharTable[Char(I)]:= I;
    end;
end;  { MakeCharTable }

function IPos(Pat, Text: PChar):Integer;
var
  RunPat, RunText, PosPtr: PChar;
begin
  Result:= 0;
  RunPat:= Pat;
  RunText:= Text;
  while RunText^ <> #0 do
  begin
    if (CharTable[RunPat^] = CharTable[RunText^]) then
      begin
        PosPtr:= RunText;
        while RunPat^ <> #0 do begin
          if (CharTable[RunPat^] <> CharTable[RunText^]) then break;
          inc(RunPat); inc(RunText);
        end;
        if RunPat^ = #0 then
          begin
            Result:= PosPtr - Text +1;
            break;
          end;
      end else inc(RunText);
      RunPat:= Pat;
  end;
end; {IPos }

function IPosE(Pat, Text: PChar; CharPos: LongInt):Integer;
var
  RunPat, RunText, PosPtr: PChar;
begin
  Result:= 0;
  RunPat:= Pat;
  RunText:= Text + CharPos -1;
  while RunText^ <> #0 do
  begin
    if (CharTable[RunPat^] = CharTable[RunText^]) then
      begin
        PosPtr:= RunText;
        while RunPat^ <> #0 do begin
          if (CharTable[RunPat^] <> CharTable[RunText^]) then break;
          inc(RunPat); inc(RunText);
        end;
        if RunPat^ = #0 then
          begin
            Result:= PosPtr - Text +1;
            break;
          end;
      end else inc(RunText);
      RunPat:= Pat;
  end;
end; {IPosE }

Initialization
  MakeCharTable
end.
